#!/bin/sh -e

. shell-error

verbose=

rpath=
basedir=
bindir=
libdir=
idldir=
includedir=

TEMP=`getopt -n $PROG -o v,h -l develdir:,bindir:,libdir:,idldir:,includedir:,rpath:,verbose,help -- "$@"` ||
	show_usage
eval set -- "$TEMP"
while :; do
	case "$1" in
		--develdir) shift
			basedir="$(readlink -ev "$1")"
			;;
		--bindir) shift
			bindir="$(readlink -ev "$1")"
			;;
		--libdir) shift
			libdir="$(readlink -ev "$1")"
			;;
		--idldir) shift
			idldir="$(readlink -ev "$1")"
			;;
		--includedir) shift
			includedir="$(readlink -ev "$1")"
			;;
		--rpath) shift
			rpath="$1"
			;;
		-v|--verbose) verbose=-v
			;;
		-h|--help) show_help
			;;
		--) shift; break
			;;
		*) fatal "Unrecognized option: $1"
			;;
	esac
	shift
done

[ -d "$basedir" ] ||
	fatal "develdir required"

[ -d "$bindir" ] ||
	fatal "bindir required"

[ -d "$libdir" ] ||
	fatal "libdir required"

[ -d "$includedir" ] ||
	fatal "includedir required"

# Fixup bin
find -P "$bindir" "$basedir" -type l -printf '%p\t%l\n' |
while read link target; do
	[ -z "${target%%/*}" ] ||
		continue

	t="$(relative "$target" "$link")"
	ln $verbose -nsf -- "$t" "$link"
done

### Fixup lib
if [ -n "$rpath" ]; then
	fake_rpath="/$(printf '%s' "$rpath" |tr '[:print:]' '_')"
	find "$libdir" "$basedir/sdk/lib" -type f | 
	while read f; do
		file "$f" | grep -qs ELF ||
			continue

		if chrpath -l "$f" |fgrep -qs "RPATH=$fake_rpath"; then
			chrpath -r "$rpath" "$f"
		fi
	done
fi

### Fixup SDK
find "$basedir/sdk/lib" -type f |
while read lib; do
	[ -e "$libdir/${lib##*/}" ] ||
		continue

	cmp -s "$libdir/${lib##*/}" "$lib" ||
		continue

	target="$(relative "$libdir/${lib##*/}" "$lib")" ||
		continue

	rm $verbose -f -- "$lib" ||:
	ln $verbose -nsf -- "$target" "$lib"
done

### Fixup headers
find -P "$includedir/unstable" -type f -name '*.h' |
while read header; do
	name="${header##*/unstable/}"

	[ -e "$includedir/stable/$name" ] ||
		continue
	
	cmp -s "$header" "$includedir/stable/$name" ||
		continue

	rm -f -- "$header"
	ln $verbose -nsf -- "../stable/$name" "$header"
done

find -P "$includedir/unstable" -type f -name '*.cfg' |
while read header; do
	name="${header##*/unstable/}"

	[ -e "$includedir/stable/md/$name" ] ||
		continue
	
	cmp -s "$header" "$includedir/stable/md/$name" ||
		continue

	rm -f -- "$header"
	ln $verbose -nsf -- "../stable/md/$name" "$header"
done

### Fixup IDL
find -P "$idldir/unstable" -type f -name '*.idl' |
while read idl; do
	name="${idl##*/unstable/}"

	[ -e "$idldir/stable/$name" ] ||
		continue
	
	cmp -s "$idl" "$idldir/stable/$name" ||
		continue

	rm -f -- "$idl"
	ln $verbose -nsf -- "../stable/$name" "$idl"
done
